

import os 
import sys
этот_файл = __file__ 
эта_папка = os.path.dirname(этот_файл) 
папка_выше = os.path.dirname(эта_папка)
sys.path.append(папка_выше) 
from проверка_элемента import проверка_содержит
from терминал import печать
import проверка_элемента



def записать_в_файл(путь : str, текст : str) -> str:
    with open(путь,"w", encoding="utf-8") as file:
        file.write(текст)



РАЗДЕЛИТЕЛЬ_ПАР = "___,___333223232№233АВЫФВ"
РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ  = "___:---2=2049в___"



def считать_из_файла(путь):
    with open(путь, "r", encoding="utf-8") as file:
        return file.read()
    





def преобразовать_словарь_в_текст(словарь :dict ):
    # если разделитель встречается в одном из значений слоаря,
    # то выдать ошибку "В одном из значений словаря содержится разделитель"
    if проверка_содержит(
        что = [РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ,РАЗДЕЛИТЕЛЬ_ПАР],
                где= list(словарь.values())
                
                
                ):

        raise ValueError (печать (" В одном из значений словаря найден разделитель ",
                         красный =True,
                         не_печатать = True 
        ))
    словарь.values()
    результат = ""
    for ключ,значение in словарь.items():
        пара_в_тексте = f"{ключ}{РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ}{значение}"
        результат += пара_в_тексте + РАЗДЕЛИТЕЛЬ_ПАР
    результат = результат[:-len(РАЗДЕЛИТЕЛЬ_ПАР)]
    return результат





def записать_словари_в_файл(путь: str, список_словарей: str) -> None:
    общий_результат =""
    for список_словарей in список_словарей:
        результат = преобразовать_словарь_в_текст(список_словарей) 
        общий_результат += результат + "\n"
        # "\n" специальный символ, означающий завершение абзаца.Все,что написанно после него - будет еаписанно с новой строки
    записать_в_файл(путь,общий_результат)
    



    
def считать_словари_из_файла(путь: str ,) -> list:
    текст = считать_из_файла(путь)
    список_словарей = []
    for строка in текст.splitlines():
        пары = строка.split(РАЗДЕЛИТЕЛЬ_ПАР)
        словарь = {}
        for пара in пары:
            ключ, значение = пара.split(РАЗДЕЛИТЕЛЬ_КЛЮЧЕЙ_ОТ_ЗНАЧЕНИЙ) 
            словарь[ключ] = значение
        список_словарей.append(словарь)
    
    return список_словарей








if __name__== "__main__":

    путь = "C:\\Users\\2024.09 Пайтон\\Desktop\Ученики\\Акимов Константин\\Dizert\\ff.txt"
    текст = "Гуглить это плохо"
# Windos разбивай разные папки с помощью дроби\ 
# питон воспринемает этот символ как специальный и ожидает после него дополнительный специальный символ
# Поэтому эсли просто скопировать путь к файлу, питон выдаст ошибку.
# Чтобы этого недопустить надо:
    # либо проставить везде двойные дроби вместо одинарных "\\"
    # либо до кавычки поставить букву "R", но тогда не получиться использовать "F" строку
    # записать_в_файл(путь,текст) 
    # текст_из_файла = считать_из_файла(путь)
    # print(текст_из_файла == текст) 



    список_словарей = [{
        "имя": "н___:___а", 
        "фамилия": "Тарасов",
        "Должность": "преподаватель",
        "возраст": "35"
    } , {"имя": "Федор", 
        "фамилия": "Черенков",
        "Должность": "футболист",
        "возраст": "42"
    } , ]

    записать_словари_в_файл(путь,список_словарей) 
    результат = считать_словари_из_файла(путь) 
    print(результат == список_словарей)
    считать_словари_из_файла(путь) 




def печать(
        текст, 
        красный=False, 
        зеленый=False, 
        синий=False, 
        желтый=False,
        стиль = None,
        не_печатать = False
): 
    def определить_стиль(стиль):
       
        if стиль == None:
         return ""

        СТИЛИ ={
            "ж":1,
            "к":3,
            "ч":4,
            "з":9,

        }

        # при комбинировании жирного и зачеркнутого дллжен получится результат
        # ;1;9
        # жирный и курсивный - ;1;3
        накопительная =""
        for буква in стиль:
            код_стиля = СТИЛИ.get(буква) 
            этот_стиль = f";{код_стиля}"
            накопительная += этот_стиль
                    
        
        
        return накопительная
    
    
    # определенные символы в начале и в конце написанны на C++ и отвечают за кодировку цвета 

    # первая часть строки задает стиль цвета и формат шрифта 
    # \x1B[ - специальный набор символов, который включает этот режим настройки 
    # 38:2; - говорит что следуйщие 3 параметра будут цветом в формате GRB 
    # 225;0;0 - формате rgb 
    # 4подчеркнутый стиль шрифта 
    # m - завершает настройку шрифта 
    # последняя часть строки сбрасывает стиль в стандарт, чтобы все дальнейшее писалось обычным цветом
    # результат = "\x1B[38;2;225;0;0;4m" + текст + "\x1B[0m"  

    готовый_стиль =определить_стиль(стиль)

    if красный:
        цвет = "255;0;0" 
    elif зеленый:
        цвет = "0;255;0" 
    elif синий:
        цвет = "0;0;255"
    elif желтый:
        цвет = "240;248;38"
    else:
        цвет = "255;255;255"
    
    
    результат = f"\x1B[38;2;{цвет}{готовый_стиль}m{текст}\x1B[0m" 
    if не_печатать:
        return результат
    print(результат) 
    
    
# если оставить строки запуском функций внутри файла с модулем, то они будут выполнены в МОМЕНТ ИМПОРТА этого модуля,чтобы этого не допустить, но при этом оставить возможность тестировать этот файл модуля "прямо здесь" например, чтобы провести его отладк, надо эти(ту) строку(и) убрать внутрь блока ЕСЛИ , который будет проверять откуда запущен этот  файл 

# ЕСЛИ файл произошел из этого файла (если он не импортируется) 
if __name__ == "__main__":

    печать("123", красный= True, стиль="жч")  

    красный = печать("красный", красный=True, не_печатать=True) 
    зеленый =печать("зеленый", зеленый=True,не_печатать=True) 
    разноцветная_строка = f"{красный} и {зеленый}" 
    print(разноцветная_строка)





# принцып програирования TDD - Test Driven Development 
# переводится как "разработка основанная на тестировании" 
# данный прнцып предлогает, что с начало создаются тесты для нужной функции, которые будут проверять все возможные варианты ввода и вывода и только потом пишется сама функция. 
# При каждом обновлении кода функции необходимо запустить все коды функции все тесты и удостовериться что все они пройдены и нет никаких ошибок 

# ТЕСТЫ ВСЕГДА ДОЛЖНЫ ПИСАТЬСЯ ЯВНЫМ ОБРАЗОМ - все входные данные и ожидаекмые результаты работы функций долдны быть написанны от руки"- т.е. они не должны вычисляться самим тестом 


def максимальное_число(список):
    # список = [5, 9, 2, 3, 15]
    
    if type(список) !=list  and type(список) != tuple:
        raise TypeError (f"Эта функция может принять только список или кортеж.по факты был передан {type(список)}")
    
    наибольшее = список[0]
    for элемент in список:
        if элемент > наибольшее:
            наибольшее = элемент
    return наибольшее


def тест():
    название_теста = "список чисел"
    данные = [5, 9, 2, 3, 15]
    ожидаемый_ответ = 15 
    результат = максимальное_число(данные) 
    if результат == ожидаемый_ответ:
        print(f'тест"{название_теста}"пройден!') 
    else:
        print(f'тест"{название_теста}"не пройден!') 
        print(f"ожидался ответ - {ожидаемый_ответ}")
        print(f"получили результат - {результат}")


def тест():
    название_теста = "список чисел"
    данные = [5, 9, 2, 3, 15, -2, -30 , 21, 0, 1, -50]
    ожидаемый_ответ = 21
    результат = максимальное_число(данные) 
    if результат == ожидаемый_ответ:
        print(f'тест"{название_теста}"пройден!') 
    else:
        print(f'тест"{название_теста}"не пройден!') 
        print(f"ожидался ответ - {ожидаемый_ответ}")
        print(f"получили результат - {результат}") 


        
def тест():
    название_теста = "передан не список"
    данные = "Какой-то текст " 
    ожидаемый_ответ = "Ошибка"
    try:
        результат = максимальное_число(данные) 
    except:
        результат = "Ошибка"
    if результат == ожидаемый_ответ:
        print(f'тест"{название_теста}"пройден!') 
    else:
        print(f'тест"{название_теста}"не пройден!') 
        print(f"ожидался ответ - {ожидаемый_ответ}")
        print(f"получили результат - {результат}")
тест()




import os 
import sys
этот_файл = __file__ 
эта_папка = os.path.dirname(этот_файл) 
папка_выше = os.path.dirname(эта_папка)
sys.path.append(папка_выше) 
from инструменты.терминал import печать

пройдено = 0

def проверка_содержит(что,где,игнорировать_регистр = False, стоп=False): 
    # функция проверяет что хотя бы один элемент из "что" (или вся переменная целиком) содержится хотя бы в одном элементе "где"(или является частью переменной "где")
    # функция возвращает True или False
    # переменные "что" и "где" могут быть 2 видов
        #1.Итеррируемые объекты (кортыж или список), всеми элементами которых являются строки(текст)
        # 2.сама переменная может быть строкой (текстом)
        # при нарушении люього из этих пунктов бутет вызвана ошибка с описание проблемы 
    # переменная "игнорировать_регистр" может принимать только True , или быть по умолчанию false.При передаче другого типа данных будет вызвана ошибка.
    # При получении в "что" или "где" пустых строк или пустых списков будет возвращаться False 
    #
    if type(что) not in [list,str,tuple]:
        raise TypeError(f'Переменная "что" может принять только список, кортеж или текст.по факту получая {type(что)}')
    if type(где) not in [list,str,tuple]:
        raise TypeError(f'Переменная "где" может принять только список, кортеж или текст.по факту получая {type(где)}')
    if type(игнорировать_регистр) != bool:
         raise TypeError(f'Переменная "игнорировать_регистр" может принять только список, кортеж или текст.по факту получая {type(игнорировать_регистр)}')

    if что in [[],(),""]:
        if что == []:
            текст_ошибки = "список"
        elif что ==():
            текст_ошибки = "Кортежем" 
        elif что == "":
            текст_ошибки = "текст"
        raise TypeError(f"Перемненная 'где' не может принять пустой {текст_ошибки}")
    
    if где in [[],(),""]:
        if где == []:
            текст_ошибки = "список"
        elif где ==():
            текст_ошибки = "Кортежем" 
        elif где == "":
            текст_ошибки = "текст"
        raise TypeError(f"Перемненная 'где' не может принять пустой {текст_ошибки}")
        
    for элемент in что:
        if type(элемент) != str:
            raise TypeError (f"Внутри переменной 'что' оказался не текстовый элемент - {элемент}") 
        
    for элемент in где:
        if type(элемент) != str:
            raise TypeError (f"Внутри переменной 'что' оказался не текстовый элемент - {элемент}") 
        
    

# новое
    if стоп:
         print('стоп')

    if игнорировать_регистр:
        if type(что) ==str:
            что = что.lower()  
        else:
            # # что = что.copy()
            # for индекс in range(len(что)):
            #     что[индекс] = что[индекс].lower()

            маленькие_буквы = []
            for элемент in что:
                маленькие_буквы.append(элемент.lower())
            что = маленькие_буквы

        if type(где) == str:    
            где = где.lower()  
        else:
            # # где = где.copy()
            # for индекс in range(len(где)):
            #     где[индекс] = где[индекс].lower()

            маленькие_буквы = []
            for элемент in где:
                маленькие_буквы.append(элемент.lower())
            где = маленькие_буквы
           

# текст - текст
    if type(что) == str and type(где) == str:
            return что in где
# текст - множество
    if type(что) == str:
        for элемент in где:
            if что in элемент:
                return True

    if type(где) == str:
        for элемент  in что:
            if элемент in где:
                return True


    if type(что) != str and type(где) != str:
        for элемент_что in что:
            for элемент_где in где:
                if элемент_что in элемент_где:
                    return True

    return False

def тесты():
    def тест(что, где,регистр,ожидаемый_ответ,название_теста):
                
        global пройдено
        
        try:
            результат = проверка_содержит(что,где,регистр) 
        except:
            результат = "Ошибка"
        if результат == ожидаемый_ответ:
            печать(f'тест"{название_теста}"пройден!', зеленый=True) 
            пройдено += 1
        else:
            печать(f'тест"{название_теста}"не пройден!', красный=True) 
            print(f"ожидался ответ - {ожидаемый_ответ}")
            print(f"получили результат - {результат}")
            print()


    название = "что список, где корректное, регистр корректный - не ожидаем ошибку" 
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название =  "что кортеж, где корректное, регистр корректный - не ожидаем ошибку"
    что = ("1","2","3")
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что текст, где корректное, регистр корректный - не ожидаем ошибку"
    что = "1"
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что другой тип, где корректное, регистр корректный - ожидаем ошибку" 
    что = 1
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 
 

    название = "что пустой список, где корректное, регистр корректный - ожидаем ошибку" 
    что = []
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что пустой кортеж, где корректное, регистр корректный - ожидаем ошибку" 
    что = ()
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 
 

    название = "что пустой текст, где корректное, регистр корректный - ожидаем ошибку" 
    что = ""
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "в что один из элементов не строка, где корректный регистр корректный - ожидаем ошибку"
    что = ["1","2",3] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 
    
    название = "что корректное, где список, регистр корректный - не ожидаем ошибку" 
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где кортеж, регистр корректный - не ожидаем ошибку" 
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где текст, регистр корректный - не ожидаем ошибку" 
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где другой тип, регистр корректный - ожидаем ошибку" 
    что = "1"
    где = 1
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где пустой список, регистр корректный - ожидаем ошибку" 
    что = "1"
    где = [] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где пустой кортеж, регистр корректный - ожидаем ошибку" 
    что = "1"
    где = ()
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где пустой текст, регистр корректный - ожидаем ошибку" 
    что = "1"
    где = ""
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, в где один элементов не строка, регистр корректный - ожидаем ошибку" 
    что = "1"
    где = [1] 
    регистор = True 
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где корректное, регистр True - не ожидаем ошибку" 
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где корректное, регистр False - не ожидаем ошибку"
    что = ["1","2","3"] 
    где = ["1","2","3"] 
    регистор = True 
    ожидаемый_ответ = True
    тест(что, где,регистор,ожидаемый_ответ, название) 

    название = "что корректное, где корректное, регистр другой тип - ожидаем ошибку" 
    что = "1"
    где = ["1","2","3"] 
    регистор = 1
    ожидаемый_ответ = "Ошибка"
    тест(что, где,регистор,ожидаемый_ответ, название) 


    видео = ".mp4"
    картинка = ".jpg"
    музыка = ".mp3"
    ворд = ".docx"
    текст = ".txt"
    книга = ".fb2"
    эксель = ".xlsx"

    список_видео = ["фильм.mp4", "кино.mp4", "песня.MP3"]
    список_картинок = ["котики.jpg", "море.jpg", "каникулы.mp4"]
    список_музыки = ["мелодия.mp3", "трек.mp3", "обложка.JPG"]
    список_документов = ["доклад.docx", "список дел.txt", "книга.fb2", "таблица.xlsx"]

    варианты = [
            ["элемент-элемент, регистр важен, результат - не содержит", ".jpg", "обложка.JPG", False, False],
            ["элемент-элемент, регистр важен, результат - содержит", ".jpg", "котики.jpg", False, True],
            ["элемент-элемент, регистр не важен, результат - не содержит", ".jpg", "песня.MP3", True, False],
            ["элемент-элемент, регистр не важен, результат - содержит", ".jpg", "обложка.JPG", True, True],
            ["элемент-множество, регистр важен, результат - не содержит", ".mp3", список_видео, False, False],
            ["элемент-множество, регистр важен, результат - содержит", ".jpg", список_картинок, False, True],
            ["элемент-множество, регистр не важен, результат - не содержит", ".txt", список_видео, True, False],
            ["элемент-множество, регистр не важен, результат - содержит", ".mp3", список_видео, True, True],
            ["множество-элемент, регистр важен, результат - не содержит", [".jpg", ".mp3"], "песня.MP3", False, False],
            ["множество-элемент, регистр важен, результат - содержит", [".txt", ".mp3"], "список дел.txt", False, True],
            ["множество-элемент, регистр не важен, результат - не содержит", [".xlsx", ".fb2"], "трек.mp3", True, False],
            ["множество-элемент, регистр не важен, результат - содержит", [".jpg", ".mp3"], "песня.MP3", True, True],
            ["множество-множество, регистр важен, результат - не содержит", [".jpg", ".mp3"], список_видео, False, False],
            ["множество-множество, регистр важен, результат - содержит", [".mp4", ".jpg"], список_картинок, False, True],
            ["множество-множество, регистр не важен, результат - не содержит", [".docx", ".xlsx"], список_видео, True, False],
            ["множество-множество, регистр не важен, результат - содержит", [".mp3", ".docx"], список_видео, True, True],
    ]


    for  вариант in варианты:
        название, что ,где ,регистор, ожидаемый_ответ = вариант
        тест(что, где,регистор,ожидаемый_ответ, название) 

    if проверка_содержит(что,где,регистор,стоп=True) == ожидаемый_ответ:
        print("верно")
    else:
        print("неверно")    
    print(f"пройдено тестов {пройдено} из 35")
    print(где)

    вариант = ["элемент-множество, регистр важен, результат - содержит", ".jpg", список_картинок, False, True]


if __name__ ==  "__main":
    тесты()


from setuptools import setup

setup(
    name= "Akimka_tools", #имя модуля которое будет использоваться при установке в будуйщем
    version="0.1", # версия модуля.при каждом обновлении(новой отправке) надо увеличивать версию 
    description="Мои инструменты", # описание. Можно не использовать 
    packages=["Akimka_001"], #Список с названием папки которые нужно залить
    author_email="akimov.konstantin.2024@gmail.com", # Почта модуля 
    zip_safe= False , # Пароль
    # install_reguires=[] ,список название модулей которые нужны для корректной работы этого модуля 
    
)